/*      */ package org.hibernate.cfg;
/*      */ 
/*      */ import java.io.File;
/*      */ import java.io.IOException;
/*      */ import java.io.InputStream;
/*      */ import java.lang.reflect.Constructor;
/*      */ import java.lang.reflect.Method;
/*      */ import java.net.URL;
/*      */ import java.util.ArrayList;
/*      */ import java.util.Comparator;
/*      */ import java.util.HashMap;
/*      */ import java.util.HashSet;
/*      */ import java.util.Iterator;
/*      */ import java.util.List;
/*      */ import java.util.Map;
/*      */ import java.util.Map.Entry;
/*      */ import java.util.Properties;
/*      */ import java.util.ResourceBundle;
/*      */ import java.util.Set;
/*      */ import java.util.SortedSet;
/*      */ import java.util.StringTokenizer;
/*      */ import java.util.TreeSet;
/*      */ import javax.persistence.Entity;
/*      */ import javax.persistence.MappedSuperclass;
/*      */ import org.apache.commons.logging.Log;
/*      */ import org.apache.commons.logging.LogFactory;
/*      */ import org.dom4j.Attribute;
/*      */ import org.dom4j.DocumentException;
/*      */ import org.dom4j.Element;
/*      */ import org.dom4j.io.SAXReader;
/*      */ import org.hibernate.AnnotationException;
/*      */ import org.hibernate.HibernateException;
/*      */ import org.hibernate.Interceptor;
/*      */ import org.hibernate.MappingException;
/*      */ import org.hibernate.SessionFactory;
/*      */ import org.hibernate.annotations.AnyMetaDef;
/*      */ import org.hibernate.annotations.common.reflection.ReflectionManager;
/*      */ import org.hibernate.annotations.common.reflection.XClass;
/*      */ import org.hibernate.cfg.annotations.Version;
/*      */ import org.hibernate.cfg.annotations.reflection.EJB3ReflectionManager;
/*      */ import org.hibernate.cfg.annotations.reflection.XMLContext;
/*      */ import org.hibernate.cfg.search.SearchConfiguration;
/*      */ import org.hibernate.event.EventListeners;
/*      */ import org.hibernate.event.PreInsertEventListener;
/*      */ import org.hibernate.event.PreUpdateEventListener;
/*      */ import org.hibernate.mapping.Column;
/*      */ import org.hibernate.mapping.Join;
/*      */ import org.hibernate.mapping.PersistentClass;
/*      */ import org.hibernate.mapping.Table;
/*      */ import org.hibernate.mapping.UniqueKey;
/*      */ import org.hibernate.mapping.Value;
/*      */ import org.hibernate.util.JoinedIterator;
/*      */ import org.hibernate.util.ReflectHelper;
/*      */ import org.hibernate.util.StringHelper;
/*      */ import org.hibernate.util.XMLHelper;
/*      */ import org.xml.sax.InputSource;
/*      */ import org.xml.sax.SAXException;
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ public class AnnotationConfiguration
/*      */   extends Configuration
/*      */ {
/*   70 */   private static Log log = LogFactory.getLog(AnnotationConfiguration.class);
/*      */   public static final String ARTEFACT = "hibernate.mapping.precedence";
/*      */   
/*   73 */   static { Version.touch(); }
/*      */   
/*      */ 
/*      */   public static final String DEFAULT_PRECEDENCE = "hbm, class";
/*      */   
/*      */   private Map namedGenerators;
/*      */   
/*      */   private Map<String, Map<String, Join>> joins;
/*      */   private Map<String, AnnotatedClassType> classTypes;
/*      */   private Set<String> defaultNamedQueryNames;
/*      */   private Set<String> defaultNamedNativeQueryNames;
/*      */   private Set<String> defaultSqlResulSetMappingNames;
/*      */   private Set<String> defaultNamedGenerators;
/*      */   private Map<String, Properties> generatorTables;
/*      */   private Map<Table, List<String[]>> tableUniqueConstraints;
/*      */   private Map<String, String> mappedByResolver;
/*      */   private Map<String, String> propertyRefResolver;
/*      */   private Map<String, AnyMetaDef> anyMetaDefs;
/*      */   private List<XClass> annotatedClasses;
/*      */   private Map<String, XClass> annotatedClassEntities;
/*      */   private Map<String, org.dom4j.Document> hbmEntities;
/*      */   private List<CacheHolder> caches;
/*      */   private List<org.dom4j.Document> hbmDocuments;
/*   96 */   private String precedence = null;
/*   97 */   private boolean inSecondPass = false;
/*      */   private transient ReflectionManager reflectionManager;
/*   99 */   private boolean isDefaultProcessed = false;
/*      */   
/*      */ 
/*      */   private boolean isValidatorNotPresentLogged;
/*      */   
/*      */ 
/*      */   public AnnotationConfiguration(SettingsFactory sf)
/*      */   {
/*  107 */     super(sf);
/*      */   }
/*      */   
/*      */   protected List<XClass> orderAndFillHierarchy(List<XClass> original)
/*      */   {
/*  112 */     List<XClass> copy = new ArrayList(original);
/*      */     
/*  114 */     for (Iterator i$ = original.iterator(); i$.hasNext(); 
/*      */         
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*  121 */         goto 45)
/*      */     {
/*  114 */       XClass clazz = (XClass)i$.next();
/*  115 */       XClass superClass = clazz.getSuperclass();
/*  116 */       if ((superClass != null) && (!this.reflectionManager.equals(superClass, Object.class)) && (!copy.contains(superClass))) {
/*  117 */         if ((superClass.isAnnotationPresent(Entity.class)) || (superClass.isAnnotationPresent(MappedSuperclass.class)))
/*      */         {
/*  119 */           copy.add(superClass);
/*      */         }
/*  121 */         superClass = superClass.getSuperclass();
/*      */       }
/*      */     }
/*  124 */     List<XClass> workingCopy = new ArrayList(copy);
/*  125 */     List<XClass> newList = new ArrayList(copy.size());
/*  126 */     while (workingCopy.size() > 0) {
/*  127 */       XClass clazz = (XClass)workingCopy.get(0);
/*  128 */       orderHierarchy(workingCopy, newList, copy, clazz);
/*      */     }
/*  130 */     return newList;
/*      */   }
/*      */   
/*      */   private void orderHierarchy(List<XClass> copy, List<XClass> newList, List<XClass> original, XClass clazz) {
/*  134 */     if ((clazz == null) || (this.reflectionManager.equals(clazz, Object.class))) { return;
/*      */     }
/*  136 */     orderHierarchy(copy, newList, original, clazz.getSuperclass());
/*  137 */     if (original.contains(clazz)) {
/*  138 */       if (!newList.contains(clazz)) {
/*  139 */         newList.add(clazz);
/*      */       }
/*  141 */       copy.remove(clazz);
/*      */     }
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   public AnnotationConfiguration addAnnotatedClass(Class persistentClass)
/*      */     throws MappingException
/*      */   {
/*  152 */     XClass persistentXClass = this.reflectionManager.toXClass(persistentClass);
/*      */     try {
/*  154 */       this.annotatedClasses.add(persistentXClass);
/*  155 */       return this;
/*      */     }
/*      */     catch (MappingException me) {
/*  158 */       log.error("Could not compile the mapping annotations", me);
/*  159 */       throw me;
/*      */     }
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   public AnnotationConfiguration addPackage(String packageName)
/*      */     throws MappingException
/*      */   {
/*  170 */     log.info("Mapping package " + packageName);
/*      */     try {
/*  172 */       AnnotationBinder.bindPackage(packageName, createExtendedMappings());
/*  173 */       return this;
/*      */     }
/*      */     catch (MappingException me) {
/*  176 */       log.error("Could not compile the mapping annotations", me);
/*  177 */       throw me;
/*      */     }
/*      */   }
/*      */   
/*      */   public ExtendedMappings createExtendedMappings() {
/*  182 */     return new ExtendedMappings(this.classes, this.collections, this.tables, this.namedQueries, this.namedSqlQueries, this.sqlResultSetMappings, this.defaultNamedQueryNames, this.defaultNamedNativeQueryNames, this.defaultSqlResulSetMappingNames, this.defaultNamedGenerators, this.imports, this.secondPasses, this.propertyReferences, this.namingStrategy, this.typeDefs, this.filterDefinitions, this.namedGenerators, this.joins, this.classTypes, this.extendsQueue, this.tableNameBinding, this.columnNameBindingPerTable, this.auxiliaryDatabaseObjects, this.generatorTables, this.tableUniqueConstraints, this.mappedByResolver, this.propertyRefResolver, this.anyMetaDefs, this.reflectionManager);
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   public void setCacheConcurrencyStrategy(String clazz, String concurrencyStrategy, String region, boolean cacheLazyProperty)
/*      */     throws MappingException
/*      */   {
/*  217 */     this.caches.add(new CacheHolder(clazz, concurrencyStrategy, region, true, cacheLazyProperty));
/*      */   }
/*      */   
/*      */   public void setCollectionCacheConcurrencyStrategy(String collectionRole, String concurrencyStrategy, String region)
/*      */     throws MappingException
/*      */   {
/*  223 */     this.caches.add(new CacheHolder(collectionRole, concurrencyStrategy, region, false, false));
/*      */   }
/*      */   
/*      */   protected void reset()
/*      */   {
/*  228 */     super.reset();
/*  229 */     this.namedGenerators = new HashMap();
/*  230 */     this.joins = new HashMap();
/*  231 */     this.classTypes = new HashMap();
/*  232 */     this.generatorTables = new HashMap();
/*  233 */     this.defaultNamedQueryNames = new HashSet();
/*  234 */     this.defaultNamedNativeQueryNames = new HashSet();
/*  235 */     this.defaultSqlResulSetMappingNames = new HashSet();
/*  236 */     this.defaultNamedGenerators = new HashSet();
/*  237 */     this.tableUniqueConstraints = new HashMap();
/*  238 */     this.mappedByResolver = new HashMap();
/*  239 */     this.propertyRefResolver = new HashMap();
/*  240 */     this.annotatedClasses = new ArrayList();
/*  241 */     this.caches = new ArrayList();
/*  242 */     this.hbmEntities = new HashMap();
/*  243 */     this.annotatedClassEntities = new HashMap();
/*  244 */     this.hbmDocuments = new ArrayList();
/*  245 */     this.namingStrategy = EJB3NamingStrategy.INSTANCE;
/*  246 */     setEntityResolver(new EJB3DTDEntityResolver());
/*  247 */     this.anyMetaDefs = new HashMap();
/*  248 */     this.reflectionManager = new EJB3ReflectionManager();
/*      */   }
/*      */   
/*      */   protected void secondPassCompile() throws MappingException
/*      */   {
/*  253 */     log.debug("Execute first pass mapping processing");
/*      */     
/*      */ 
/*  256 */     List<XClass> tempAnnotatedClasses = new ArrayList(this.annotatedClasses.size());
/*  257 */     for (XClass clazz : this.annotatedClasses) {
/*  258 */       if (clazz.isAnnotationPresent(Entity.class)) {
/*  259 */         this.annotatedClassEntities.put(clazz.getName(), clazz);
/*  260 */         tempAnnotatedClasses.add(clazz);
/*      */       }
/*  262 */       else if (clazz.isAnnotationPresent(MappedSuperclass.class)) {
/*  263 */         tempAnnotatedClasses.add(clazz);
/*      */       }
/*      */     }
/*      */     
/*  267 */     this.annotatedClasses = tempAnnotatedClasses;
/*      */     
/*      */ 
/*      */ 
/*  271 */     if (!this.isDefaultProcessed) {
/*  272 */       AnnotationBinder.bindDefaults(createExtendedMappings());
/*  273 */       this.isDefaultProcessed = true;
/*      */     }
/*      */     
/*      */ 
/*  277 */     if (this.precedence == null) this.precedence = getProperties().getProperty("hibernate.mapping.precedence");
/*  278 */     if (this.precedence == null) this.precedence = "hbm, class";
/*  279 */     StringTokenizer precedences = new StringTokenizer(this.precedence, ",; ", false);
/*  280 */     if (!precedences.hasMoreElements()) {
/*  281 */       throw new MappingException("hibernate.mapping.precedence cannot be empty: " + this.precedence);
/*      */     }
/*  283 */     while (precedences.hasMoreElements()) {
/*  284 */       String artifact = (String)precedences.nextElement();
/*  285 */       removeConflictedArtifact(artifact);
/*  286 */       processArtifactsOfType(artifact);
/*      */     }
/*      */     
/*  289 */     int cacheNbr = this.caches.size();
/*  290 */     for (int index = 0; index < cacheNbr; index++) {
/*  291 */       CacheHolder cacheHolder = (CacheHolder)this.caches.get(index);
/*  292 */       if (cacheHolder.isClass) {
/*  293 */         super.setCacheConcurrencyStrategy(cacheHolder.role, cacheHolder.usage, cacheHolder.region, cacheHolder.cacheLazy);
/*      */ 
/*      */       }
/*      */       else
/*      */       {
/*  298 */         super.setCollectionCacheConcurrencyStrategy(cacheHolder.role, cacheHolder.usage, cacheHolder.region);
/*      */       }
/*      */     }
/*  301 */     this.caches.clear();
/*      */     try {
/*  303 */       this.inSecondPass = true;
/*  304 */       processFkSecondPassInOrder();
/*  305 */       Iterator iter = this.secondPasses.iterator();
/*  306 */       while (iter.hasNext()) {
/*  307 */         SecondPass sp = (SecondPass)iter.next();
/*      */         
/*  309 */         if ((sp instanceof CreateKeySecondPass)) {
/*  310 */           sp.doSecondPass(this.classes);
/*  311 */           iter.remove();
/*      */         }
/*      */       }
/*      */       
/*  315 */       iter = this.secondPasses.iterator();
/*  316 */       while (iter.hasNext()) {
/*  317 */         SecondPass sp = (SecondPass)iter.next();
/*      */         
/*  319 */         if ((sp instanceof SecondaryTableSecondPass)) {
/*  320 */           sp.doSecondPass(this.classes);
/*  321 */           iter.remove();
/*      */         }
/*      */       }
/*  324 */       super.secondPassCompile();
/*  325 */       this.inSecondPass = false;
/*      */     }
/*      */     catch (RecoverableException e)
/*      */     {
/*  329 */       throw ((RuntimeException)e.getCause());
/*      */     }
/*  331 */     Iterator tables = this.tableUniqueConstraints.entrySet().iterator();
/*      */     
/*      */     Table table;
/*      */     
/*      */     int uniqueIndexPerTable;
/*  336 */     while (tables.hasNext()) {
/*  337 */       Map.Entry entry = (Map.Entry)tables.next();
/*  338 */       table = (Table)entry.getKey();
/*  339 */       List<String[]> uniqueConstraints = (List)entry.getValue();
/*  340 */       uniqueIndexPerTable = 0;
/*  341 */       for (String[] columnNames : uniqueConstraints) {
/*  342 */         String keyName = "key" + uniqueIndexPerTable++;
/*  343 */         buildUniqueKeyFromColumnNames(columnNames, table, keyName);
/*      */       }
/*      */     }
/*  346 */     boolean applyOnDdl = getProperties().getProperty("hibernate.validator.apply_to_ddl", "true").equalsIgnoreCase("true");
/*      */     
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*  352 */     Constructor validatorCtr = null;
/*  353 */     Method applyMethod = null;
/*      */     try {
/*  355 */       Class classValidator = ReflectHelper.classForName("org.hibernate.validator.ClassValidator", getClass());
/*  356 */       Class messageInterpolator = ReflectHelper.classForName("org.hibernate.validator.MessageInterpolator", getClass());
/*  357 */       validatorCtr = classValidator.getDeclaredConstructor(new Class[] { Class.class, ResourceBundle.class, messageInterpolator, Map.class, ReflectionManager.class });
/*      */       
/*      */ 
/*      */ 
/*  361 */       applyMethod = classValidator.getMethod("apply", new Class[] { PersistentClass.class });
/*      */     }
/*      */     catch (ClassNotFoundException e) {
/*  364 */       if (!this.isValidatorNotPresentLogged) {
/*  365 */         log.info("Hibernate Validator not found: ignoring");
/*      */       }
/*  367 */       this.isValidatorNotPresentLogged = true;
/*      */     }
/*      */     catch (NoSuchMethodException e) {
/*  370 */       throw new AnnotationException(e);
/*      */     }
/*  372 */     if ((applyMethod != null) && (applyOnDdl)) {
/*  373 */       for (PersistentClass persistentClazz : this.classes.values())
/*      */       {
/*  375 */         String className = persistentClazz.getClassName();
/*  376 */         if (StringHelper.isNotEmpty(className)) {
/*      */           try {
/*  378 */             Object validator = validatorCtr.newInstance(new Object[] { ReflectHelper.classForName(className), null, null, null, this.reflectionManager });
/*      */             
/*      */ 
/*  381 */             applyMethod.invoke(validator, new Object[] { persistentClazz });
/*      */           }
/*      */           catch (Exception e) {
/*  384 */             log.warn("Unable to apply constraints on DDL for " + className, e);
/*      */           }
/*      */         }
/*      */       }
/*      */     }
/*      */   }
/*      */   
/*      */   private void processFkSecondPassInOrder() {
/*  392 */     log.debug("processing fk mappings (*ToOne and JoinedSubclass)");
/*  393 */     Iterator iter = this.secondPasses.iterator();
/*      */     
/*      */ 
/*      */ 
/*      */ 
/*  398 */     SortedSet<FkSecondPass> fkSecondPasses = new TreeSet(new Comparator()
/*      */     {
/*      */       public int compare(FkSecondPass f1, FkSecondPass f2)
/*      */       {
/*  402 */         int compare = f1.getValue().getTable().getQuotedName().compareTo(f2.getValue().getTable().getQuotedName());
/*      */         
/*      */ 
/*  405 */         if (compare == 0)
/*      */         {
/*  407 */           if (f1.hashCode() < f2.hashCode()) {
/*  408 */             compare = -1;
/*      */           }
/*  410 */           else if (f1.hashCode() == f2.hashCode()) {
/*  411 */             compare = 0;
/*      */           }
/*      */           else {
/*  414 */             compare = 1;
/*      */           }
/*      */         }
/*  417 */         return compare;
/*      */       }
/*      */     });
/*      */     
/*  421 */     while (iter.hasNext()) {
/*  422 */       SecondPass sp = (SecondPass)iter.next();
/*      */       
/*  424 */       if ((sp instanceof FkSecondPass)) {
/*  425 */         fkSecondPasses.add((FkSecondPass)sp);
/*  426 */         iter.remove();
/*      */       }
/*      */     }
/*  429 */     if (fkSecondPasses.size() > 0) {
/*  430 */       Map<String, Set<String>> isADependencyOf = new HashMap();
/*  431 */       List orderedFkSecondPasses = new ArrayList(fkSecondPasses.size());
/*  432 */       List endOfQueueFkSecondPasses = new ArrayList(fkSecondPasses.size());
/*  433 */       List orderedTable = new ArrayList(fkSecondPasses.size());
/*  434 */       Iterator it = fkSecondPasses.iterator();
/*  435 */       while (it.hasNext()) {
/*  436 */         FkSecondPass sp = (FkSecondPass)it.next();
/*  437 */         String referenceEntityName = sp.getReferencedEntityName();
/*  438 */         PersistentClass classMapping = getClassMapping(referenceEntityName);
/*  439 */         if (sp.isInPrimaryKey()) {
/*  440 */           String dependentTable = classMapping.getTable().getQuotedName();
/*  441 */           if (!isADependencyOf.containsKey(dependentTable)) {
/*  442 */             isADependencyOf.put(dependentTable, new HashSet());
/*      */           }
/*  444 */           String table = sp.getValue().getTable().getQuotedName();
/*  445 */           ((Set)isADependencyOf.get(dependentTable)).add(table);
/*  446 */           int beAfter = orderedTable.indexOf(dependentTable);
/*  447 */           int beBefore = orderedFkSecondPasses.size();
/*  448 */           Set<String> dependencies = (Set)isADependencyOf.get(table);
/*  449 */           if (dependencies != null) {
/*  450 */             for (String tableDep : dependencies)
/*      */             {
/*  452 */               int index = orderedTable.indexOf(tableDep);
/*      */               
/*  454 */               beBefore = (index != -1) && (index < beBefore) ? index : beBefore;
/*      */             }
/*      */           }
/*  457 */           int currentIndex = orderedTable.indexOf(table);
/*  458 */           if ((beBefore < beAfter) || ((currentIndex != -1) && ((currentIndex < beAfter) || (currentIndex > beBefore))))
/*      */           {
/*      */ 
/*  461 */             StringBuilder sb = new StringBuilder("Foreign key circularity dependency involving the following tables: ");
/*      */             
/*      */ 
/*      */ 
/*  465 */             sb.append(table);
/*  466 */             if (beAfter > -1) sb.append(", ").append(dependentTable);
/*  467 */             if (beBefore < orderedFkSecondPasses.size()) {
/*  468 */               sb.append(", ").append(orderedTable.get(beBefore));
/*      */             }
/*  470 */             throw new AnnotationException(sb.toString());
/*      */           }
/*  472 */           currentIndex = currentIndex == -1 ? beBefore : currentIndex;
/*  473 */           orderedTable.add(currentIndex, table);
/*  474 */           orderedFkSecondPasses.add(currentIndex, sp);
/*      */         }
/*      */         else {
/*  477 */           endOfQueueFkSecondPasses.add(sp);
/*      */         }
/*      */       }
/*  480 */       it = orderedFkSecondPasses.listIterator();
/*  481 */       while (it.hasNext()) {
/*  482 */         ((SecondPass)it.next()).doSecondPass(this.classes);
/*      */       }
/*      */       
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*  491 */       boolean stopProcess = false;
/*  492 */       RuntimeException originalException = null;
/*  493 */       while (!stopProcess) {
/*  494 */         List failingSecondPasses = new ArrayList();
/*  495 */         it = endOfQueueFkSecondPasses.listIterator();
/*  496 */         while (it.hasNext()) {
/*  497 */           SecondPass pass = (SecondPass)it.next();
/*      */           try {
/*  499 */             pass.doSecondPass(this.classes);
/*      */           }
/*      */           catch (RecoverableException e) {
/*  502 */             failingSecondPasses.add(pass);
/*  503 */             if (originalException == null) originalException = (RuntimeException)e.getCause();
/*      */           }
/*      */         }
/*  506 */         stopProcess = (failingSecondPasses.size() == 0) || (failingSecondPasses.size() == endOfQueueFkSecondPasses.size());
/*  507 */         endOfQueueFkSecondPasses = failingSecondPasses;
/*      */       }
/*  509 */       if (endOfQueueFkSecondPasses.size() > 0) {
/*  510 */         throw originalException;
/*      */       }
/*      */     }
/*      */   }
/*      */   
/*      */   private void processArtifactsOfType(String artifact) {
/*  516 */     if ("hbm".equalsIgnoreCase(artifact)) {
/*  517 */       log.debug("Process hbm files");
/*  518 */       for (org.dom4j.Document document : this.hbmDocuments) {
/*  519 */         super.add(document);
/*      */       }
/*  521 */       this.hbmDocuments.clear();
/*  522 */       this.hbmEntities.clear();
/*      */     }
/*  524 */     else if ("class".equalsIgnoreCase(artifact)) {
/*  525 */       log.debug("Process annotated classes");
/*      */       
/*  527 */       List<XClass> orderedClasses = orderAndFillHierarchy(this.annotatedClasses);
/*  528 */       Map<XClass, InheritanceState> inheritanceStatePerClass = AnnotationBinder.buildInheritanceStates(orderedClasses, this.reflectionManager);
/*      */       
/*      */ 
/*  531 */       ExtendedMappings mappings = createExtendedMappings();
/*  532 */       for (XClass clazz : orderedClasses)
/*      */       {
/*  534 */         AnnotationBinder.bindClass(clazz, inheritanceStatePerClass, mappings);
/*      */       }
/*  536 */       this.annotatedClasses.clear();
/*  537 */       this.annotatedClassEntities.clear();
/*      */     }
/*      */     else {
/*  540 */       log.warn("Unknown artifact: " + artifact);
/*      */     }
/*      */   }
/*      */   
/*      */   private void removeConflictedArtifact(String artifact) {
/*  545 */     if ("hbm".equalsIgnoreCase(artifact)) {
/*  546 */       for (String entity : this.hbmEntities.keySet()) {
/*  547 */         if (this.annotatedClassEntities.containsKey(entity)) {
/*  548 */           this.annotatedClasses.remove(this.annotatedClassEntities.get(entity));
/*  549 */           this.annotatedClassEntities.remove(entity);
/*      */         }
/*      */         
/*      */       }
/*  553 */     } else if ("class".equalsIgnoreCase(artifact)) {
/*  554 */       for (String entity : this.annotatedClassEntities.keySet()) {
/*  555 */         if (this.hbmEntities.containsKey(entity)) {
/*  556 */           this.hbmDocuments.remove(this.hbmEntities.get(entity));
/*  557 */           this.hbmEntities.remove(entity);
/*      */         }
/*      */       }
/*      */     }
/*      */   }
/*      */   
/*      */   private void buildUniqueKeyFromColumnNames(String[] columnNames, Table table, String keyName)
/*      */   {
/*  565 */     int size = columnNames.length;
/*  566 */     Column[] columns = new Column[size];
/*  567 */     Set<Column> unbound = new HashSet();
/*  568 */     Set<Column> unboundNoLogical = new HashSet();
/*  569 */     ExtendedMappings mappings = createExtendedMappings();
/*  570 */     for (int index = 0; index < size; index++) {
/*      */       try
/*      */       {
/*  573 */         String columnName = mappings.getPhysicalColumnName(columnNames[index], table);
/*  574 */         columns[index] = new Column(columnName);
/*  575 */         unbound.add(columns[index]);
/*      */       }
/*      */       catch (MappingException e)
/*      */       {
/*  579 */         unboundNoLogical.add(new Column(columnNames[index]));
/*      */       }
/*      */     }
/*  582 */     for (Column column : columns) {
/*  583 */       if (table.containsColumn(column)) {
/*  584 */         UniqueKey uc = table.getOrCreateUniqueKey(keyName);
/*  585 */         uc.addColumn(table.getColumn(column));
/*  586 */         unbound.remove(column);
/*      */       }
/*      */     }
/*  589 */     if ((unbound.size() > 0) || (unboundNoLogical.size() > 0)) {
/*  590 */       StringBuilder sb = new StringBuilder("Unable to create unique key constraint (");
/*  591 */       for (String columnName : columnNames) {
/*  592 */         sb.append(columnName).append(", ");
/*      */       }
/*  594 */       sb.setLength(sb.length() - 2);
/*  595 */       sb.append(") on table ").append(table.getName()).append(": ");
/*  596 */       for (Column column : unbound) {
/*  597 */         sb.append(column.getName()).append(", ");
/*      */       }
/*  599 */       for (Column column : unboundNoLogical) {
/*  600 */         sb.append(column.getName()).append(", ");
/*      */       }
/*  602 */       sb.setLength(sb.length() - 2);
/*  603 */       sb.append(" not found");
/*  604 */       throw new AnnotationException(sb.toString());
/*      */     }
/*      */   }
/*      */   
/*      */   protected void parseMappingElement(Element subelement, String name)
/*      */   {
/*  610 */     Attribute rsrc = subelement.attribute("resource");
/*  611 */     Attribute file = subelement.attribute("file");
/*  612 */     Attribute jar = subelement.attribute("jar");
/*  613 */     Attribute pckg = subelement.attribute("package");
/*  614 */     Attribute clazz = subelement.attribute("class");
/*  615 */     if (rsrc != null) {
/*  616 */       log.debug(name + "<-" + rsrc);
/*  617 */       addResource(rsrc.getValue());
/*      */     }
/*  619 */     else if (jar != null) {
/*  620 */       log.debug(name + "<-" + jar);
/*  621 */       addJar(new File(jar.getValue()));
/*      */     }
/*  623 */     else if (file != null) {
/*  624 */       log.debug(name + "<-" + file);
/*  625 */       addFile(file.getValue());
/*      */     }
/*  627 */     else if (pckg != null) {
/*  628 */       log.debug(name + "<-" + pckg);
/*  629 */       addPackage(pckg.getValue());
/*      */     }
/*  631 */     else if (clazz != null) {
/*  632 */       log.debug(name + "<-" + clazz);
/*  633 */       Class loadedClass = null;
/*      */       try {
/*  635 */         loadedClass = ReflectHelper.classForName(clazz.getValue());
/*      */       }
/*      */       catch (ClassNotFoundException cnf) {
/*  638 */         throw new MappingException("Unable to load class declared as <mapping class=\"" + clazz.getValue() + "\"/> in the configuration:", cnf);
/*      */ 
/*      */       }
/*      */       catch (NoClassDefFoundError ncdf)
/*      */       {
/*      */ 
/*  644 */         throw new MappingException("Unable to load class declared as <mapping class=\"" + clazz.getValue() + "\"/> in the configuration:", ncdf);
/*      */       }
/*      */       
/*      */ 
/*      */ 
/*      */ 
/*  650 */       addAnnotatedClass(loadedClass);
/*      */     }
/*      */     else {
/*  653 */       throw new MappingException("<mapping> element in configuration specifies no attributes");
/*      */     }
/*      */   }
/*      */   
/*      */   protected void add(org.dom4j.Document doc) throws MappingException
/*      */   {
/*  659 */     boolean ejb3Xml = "entity-mappings".equals(doc.getRootElement().getName());
/*  660 */     if (this.inSecondPass)
/*      */     {
/*  662 */       if (!ejb3Xml) { super.add(doc);
/*      */       }
/*      */     }
/*  665 */     else if (!ejb3Xml) {
/*  666 */       Element hmNode = doc.getRootElement();
/*  667 */       Attribute packNode = hmNode.attribute("package");
/*  668 */       String defaultPackage = packNode != null ? packNode.getValue() : "";
/*      */       
/*      */ 
/*  671 */       Set<String> entityNames = new HashSet();
/*  672 */       findClassNames(defaultPackage, hmNode, entityNames);
/*  673 */       for (String entity : entityNames) {
/*  674 */         this.hbmEntities.put(entity, doc);
/*      */       }
/*  676 */       this.hbmDocuments.add(doc);
/*      */     }
/*      */     else {
/*  679 */       List<String> classnames = ((EJB3ReflectionManager)this.reflectionManager).getXMLContext().addDocument(doc);
/*  680 */       for (String classname : classnames) {
/*      */         try {
/*  682 */           this.annotatedClasses.add(this.reflectionManager.classForName(classname, getClass()));
/*      */         }
/*      */         catch (ClassNotFoundException e) {
/*  685 */           throw new AnnotationException("Unable to load class defined in XML: " + classname, e);
/*      */         }
/*      */       }
/*      */     }
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   private static void findClassNames(String defaultPackage, Element startNode, Set names)
/*      */   {
/*  698 */     Iterator[] classes = new Iterator[4];
/*  699 */     classes[0] = startNode.elementIterator("class");
/*  700 */     classes[1] = startNode.elementIterator("subclass");
/*  701 */     classes[2] = startNode.elementIterator("joined-subclass");
/*  702 */     classes[3] = startNode.elementIterator("union-subclass");
/*      */     
/*  704 */     Iterator classIterator = new JoinedIterator(classes);
/*  705 */     while (classIterator.hasNext()) {
/*  706 */       Element element = (Element)classIterator.next();
/*  707 */       String entityName = element.attributeValue("entity-name");
/*  708 */       if (entityName == null) entityName = getClassName(element.attribute("name"), defaultPackage);
/*  709 */       names.add(entityName);
/*  710 */       findClassNames(defaultPackage, element, names);
/*      */     }
/*      */   }
/*      */   
/*      */   private static String getClassName(Attribute name, String defaultPackage) {
/*  715 */     if (name == null) return null;
/*  716 */     String unqualifiedName = name.getValue();
/*  717 */     if (unqualifiedName == null) return null;
/*  718 */     if ((unqualifiedName.indexOf('.') < 0) && (defaultPackage != null)) {
/*  719 */       return defaultPackage + '.' + unqualifiedName;
/*      */     }
/*  721 */     return unqualifiedName;
/*      */   }
/*      */   
/*      */ 
/*  725 */   public void setPrecedence(String precedence) { this.precedence = precedence; }
/*      */   
/*      */   private static class CacheHolder { public String role;
/*      */     public String usage;
/*      */     
/*  730 */     public CacheHolder(String role, String usage, String region, boolean isClass, boolean cacheLazy) { this.role = role;
/*  731 */       this.usage = usage;
/*  732 */       this.region = region;
/*  733 */       this.isClass = isClass;
/*  734 */       this.cacheLazy = cacheLazy;
/*      */     }
/*      */     
/*      */     public String region;
/*      */     public boolean isClass;
/*      */     public boolean cacheLazy;
/*      */   }
/*      */   
/*      */   public AnnotationConfiguration addInputStream(InputStream xmlInputStream)
/*      */     throws MappingException
/*      */   {
/*      */     try
/*      */     {
/*  747 */       List errors = new ArrayList();
/*  748 */       SAXReader saxReader = this.xmlHelper.createSAXReader("XML InputStream", errors, getEntityResolver());
/*      */       try {
/*  750 */         saxReader.setFeature("http://apache.org/xml/features/validation/schema", true);
/*      */         
/*      */ 
/*  753 */         saxReader.setProperty("http://apache.org/xml/properties/schema/external-schemaLocation", "http://java.sun.com/xml/ns/persistence/orm orm_1_0.xsd");
/*      */ 
/*      */       }
/*      */       catch (SAXException e)
/*      */       {
/*      */ 
/*  759 */         saxReader.setValidation(false);
/*      */       }
/*  761 */       org.dom4j.Document doc = saxReader.read(new InputSource(xmlInputStream));
/*      */       
/*      */ 
/*  764 */       if (errors.size() != 0) {
/*  765 */         throw new MappingException("invalid mapping", (Throwable)errors.get(0));
/*      */       }
/*  767 */       add(doc);
/*  768 */       return this;
/*      */     }
/*      */     catch (DocumentException e) {
/*  771 */       throw new MappingException("Could not parse mapping document in input stream", e);
/*      */     }
/*      */     finally {
/*      */       try {
/*  775 */         xmlInputStream.close();
/*      */       }
/*      */       catch (IOException ioe) {
/*  778 */         log.warn("Could not close input stream", ioe);
/*      */       }
/*      */     }
/*      */   }
/*      */   
/*      */   public SessionFactory buildSessionFactory() throws HibernateException
/*      */   {
/*  785 */     boolean enableValidatorListeners = !"false".equalsIgnoreCase(getProperty("hibernate.validator.autoregister_listeners"));
/*  786 */     Class validateEventListenerClass = null;
/*      */     try {
/*  788 */       validateEventListenerClass = ReflectHelper.classForName("org.hibernate.validator.event.ValidateEventListener", AnnotationConfiguration.class);
/*      */ 
/*      */     }
/*      */     catch (ClassNotFoundException e)
/*      */     {
/*      */ 
/*  794 */       log.debug("Validator not present in classpath, ignoring event listener registration");
/*      */     }
/*  796 */     if ((enableValidatorListeners) && (validateEventListenerClass != null))
/*      */     {
/*      */       Object validateEventListener;
/*      */       try {
/*  800 */         validateEventListener = validateEventListenerClass.newInstance();
/*      */       }
/*      */       catch (Exception e) {
/*  803 */         throw new AnnotationException("Unable to load Validator event listener", e);
/*      */       }
/*      */       
/*  806 */       boolean present = false;
/*  807 */       PreInsertEventListener[] listeners = getEventListeners().getPreInsertEventListeners();
/*  808 */       if (listeners != null) {
/*  809 */         for (Object eventListener : listeners)
/*      */         {
/*  811 */           present = (present) || (validateEventListenerClass == eventListener.getClass());
/*      */         }
/*  813 */         if (!present) {
/*  814 */           int length = listeners.length + 1;
/*  815 */           PreInsertEventListener[] newListeners = new PreInsertEventListener[length];
/*  816 */           for (int i = 0; i < length - 1; i++) {
/*  817 */             newListeners[i] = listeners[i];
/*      */           }
/*  819 */           newListeners[(length - 1)] = ((PreInsertEventListener)validateEventListener);
/*  820 */           getEventListeners().setPreInsertEventListeners(newListeners);
/*      */         }
/*      */       }
/*      */       else {
/*  824 */         getEventListeners().setPreInsertEventListeners(new PreInsertEventListener[] { (PreInsertEventListener)validateEventListener });
/*      */       }
/*      */       
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*  832 */       boolean present = false;
/*  833 */       PreUpdateEventListener[] listeners = getEventListeners().getPreUpdateEventListeners();
/*  834 */       if (listeners != null) {
/*  835 */         for (Object eventListener : listeners)
/*      */         {
/*  837 */           present = (present) || (validateEventListenerClass == eventListener.getClass());
/*      */         }
/*  839 */         if (!present) {
/*  840 */           int length = listeners.length + 1;
/*  841 */           PreUpdateEventListener[] newListeners = new PreUpdateEventListener[length];
/*  842 */           for (int i = 0; i < length - 1; i++) {
/*  843 */             newListeners[i] = listeners[i];
/*      */           }
/*  845 */           newListeners[(length - 1)] = ((PreUpdateEventListener)validateEventListener);
/*  846 */           getEventListeners().setPreUpdateEventListeners(newListeners);
/*      */         }
/*      */       }
/*      */       else {
/*  850 */         getEventListeners().setPreUpdateEventListeners(new PreUpdateEventListener[] { (PreUpdateEventListener)validateEventListener });
/*      */       }
/*      */     }
/*      */     
/*      */ 
/*      */ 
/*  856 */     SearchConfiguration.enableHibernateSearch(getEventListeners(), getProperties());
/*      */     
/*      */ 
/*  859 */     return super.buildSessionFactory();
/*      */   }
/*      */   
/*      */   public AnnotationConfiguration addFile(String xmlFile) throws MappingException
/*      */   {
/*  864 */     super.addFile(xmlFile);
/*  865 */     return this;
/*      */   }
/*      */   
/*      */   public AnnotationConfiguration addFile(File xmlFile) throws MappingException
/*      */   {
/*  870 */     super.addFile(xmlFile);
/*  871 */     return this;
/*      */   }
/*      */   
/*      */   public AnnotationConfiguration addCacheableFile(File xmlFile) throws MappingException
/*      */   {
/*  876 */     super.addCacheableFile(xmlFile);
/*  877 */     return this;
/*      */   }
/*      */   
/*      */   public AnnotationConfiguration addCacheableFile(String xmlFile) throws MappingException
/*      */   {
/*  882 */     super.addCacheableFile(xmlFile);
/*  883 */     return this;
/*      */   }
/*      */   
/*      */   public AnnotationConfiguration addXML(String xml) throws MappingException
/*      */   {
/*  888 */     super.addXML(xml);
/*  889 */     return this;
/*      */   }
/*      */   
/*      */   public AnnotationConfiguration addURL(URL url) throws MappingException
/*      */   {
/*  894 */     super.addURL(url);
/*  895 */     return this;
/*      */   }
/*      */   
/*      */   public AnnotationConfiguration addResource(String resourceName, ClassLoader classLoader) throws MappingException
/*      */   {
/*  900 */     super.addResource(resourceName, classLoader);
/*  901 */     return this;
/*      */   }
/*      */   
/*      */   public AnnotationConfiguration addDocument(org.w3c.dom.Document doc) throws MappingException
/*      */   {
/*  906 */     super.addDocument(doc);
/*  907 */     return this;
/*      */   }
/*      */   
/*      */   public AnnotationConfiguration addResource(String resourceName) throws MappingException
/*      */   {
/*  912 */     super.addResource(resourceName);
/*  913 */     return this;
/*      */   }
/*      */   
/*      */   public AnnotationConfiguration addClass(Class persistentClass) throws MappingException
/*      */   {
/*  918 */     super.addClass(persistentClass);
/*  919 */     return this;
/*      */   }
/*      */   
/*      */   public AnnotationConfiguration addJar(File jar) throws MappingException
/*      */   {
/*  924 */     super.addJar(jar);
/*  925 */     return this;
/*      */   }
/*      */   
/*      */   public AnnotationConfiguration addDirectory(File dir) throws MappingException
/*      */   {
/*  930 */     super.addDirectory(dir);
/*  931 */     return this;
/*      */   }
/*      */   
/*      */   public AnnotationConfiguration setInterceptor(Interceptor interceptor)
/*      */   {
/*  936 */     super.setInterceptor(interceptor);
/*  937 */     return this;
/*      */   }
/*      */   
/*      */   public AnnotationConfiguration setProperties(Properties properties)
/*      */   {
/*  942 */     super.setProperties(properties);
/*  943 */     return this;
/*      */   }
/*      */   
/*      */   public AnnotationConfiguration addProperties(Properties extraProperties)
/*      */   {
/*  948 */     super.addProperties(extraProperties);
/*  949 */     return this;
/*      */   }
/*      */   
/*      */   public AnnotationConfiguration mergeProperties(Properties properties)
/*      */   {
/*  954 */     super.mergeProperties(properties);
/*  955 */     return this;
/*      */   }
/*      */   
/*      */   public AnnotationConfiguration setProperty(String propertyName, String value)
/*      */   {
/*  960 */     super.setProperty(propertyName, value);
/*  961 */     return this;
/*      */   }
/*      */   
/*      */   public AnnotationConfiguration configure() throws HibernateException
/*      */   {
/*  966 */     super.configure();
/*  967 */     return this;
/*      */   }
/*      */   
/*      */   public AnnotationConfiguration configure(String resource) throws HibernateException
/*      */   {
/*  972 */     super.configure(resource);
/*  973 */     return this;
/*      */   }
/*      */   
/*      */   public AnnotationConfiguration configure(URL url) throws HibernateException
/*      */   {
/*  978 */     super.configure(url);
/*  979 */     return this;
/*      */   }
/*      */   
/*      */   public AnnotationConfiguration configure(File configFile) throws HibernateException
/*      */   {
/*  984 */     super.configure(configFile);
/*  985 */     return this;
/*      */   }
/*      */   
/*      */   protected AnnotationConfiguration doConfigure(InputStream stream, String resourceName) throws HibernateException
/*      */   {
/*  990 */     super.doConfigure(stream, resourceName);
/*  991 */     return this;
/*      */   }
/*      */   
/*      */   public AnnotationConfiguration configure(org.w3c.dom.Document document) throws HibernateException
/*      */   {
/*  996 */     super.configure(document);
/*  997 */     return this;
/*      */   }
/*      */   
/*      */   protected AnnotationConfiguration doConfigure(org.dom4j.Document doc) throws HibernateException
/*      */   {
/* 1002 */     super.doConfigure(doc);
/* 1003 */     return this;
/*      */   }
/*      */   
/*      */   public AnnotationConfiguration setCacheConcurrencyStrategy(String clazz, String concurrencyStrategy) throws MappingException
/*      */   {
/* 1008 */     super.setCacheConcurrencyStrategy(clazz, concurrencyStrategy);
/* 1009 */     return this;
/*      */   }
/*      */   
/*      */   public AnnotationConfiguration setCollectionCacheConcurrencyStrategy(String collectionRole, String concurrencyStrategy) throws MappingException
/*      */   {
/* 1014 */     super.setCollectionCacheConcurrencyStrategy(collectionRole, concurrencyStrategy);
/* 1015 */     return this;
/*      */   }
/*      */   
/*      */   public AnnotationConfiguration setNamingStrategy(NamingStrategy namingStrategy)
/*      */   {
/* 1020 */     super.setNamingStrategy(namingStrategy);
/* 1021 */     return this;
/*      */   }
/*      */   
/*      */   public ReflectionManager getReflectionManager()
/*      */   {
/* 1026 */     return this.reflectionManager;
/*      */   }
/*      */   
/*      */   public AnnotationConfiguration() {}
/*      */ }


/* Location:              C:\Users\anandkum\Desktop\source\hibernate-annotations-3.3.1.GA.jar!\org\hibernate\cfg\AnnotationConfiguration.class
 * Java compiler version: 5 (49.0)
 * JD-Core Version:       0.7.1
 */