/*     */ package org.hibernate.ejb.packaging;
/*     */ 
/*     */ import java.io.IOException;
/*     */ import java.io.InputStream;
/*     */ import java.net.URL;
/*     */ import java.net.URLConnection;
/*     */ import java.util.ArrayList;
/*     */ import java.util.List;
/*     */ import java.util.Map;
/*     */ import java.util.Properties;
/*     */ import java.util.Set;
/*     */ import javax.persistence.PersistenceException;
/*     */ import javax.persistence.spi.PersistenceUnitTransactionType;
/*     */ import javax.xml.parsers.DocumentBuilder;
/*     */ import javax.xml.parsers.DocumentBuilderFactory;
/*     */ import org.apache.commons.logging.Log;
/*     */ import org.apache.commons.logging.LogFactory;
/*     */ import org.hibernate.cfg.EJB3DTDEntityResolver;
/*     */ import org.hibernate.ejb.util.ConfigurationHelper;
/*     */ import org.hibernate.util.StringHelper;
/*     */ import org.w3c.dom.Document;
/*     */ import org.w3c.dom.Element;
/*     */ import org.w3c.dom.Node;
/*     */ import org.w3c.dom.NodeList;
/*     */ import org.xml.sax.EntityResolver;
/*     */ import org.xml.sax.ErrorHandler;
/*     */ import org.xml.sax.InputSource;
/*     */ import org.xml.sax.SAXParseException;
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ public final class PersistenceXmlLoader
/*     */ {
/*  38 */   private static Log log = LogFactory.getLog(PersistenceXmlLoader.class);
/*     */   
/*     */ 
/*     */   private static Document loadURL(URL configURL, EntityResolver resolver)
/*     */     throws Exception
/*     */   {
/*  44 */     InputStream is = null;
/*  45 */     if (configURL != null) {
/*  46 */       URLConnection conn = configURL.openConnection();
/*  47 */       conn.setUseCaches(false);
/*  48 */       is = conn.getInputStream();
/*     */     }
/*  50 */     if (is == null) {
/*  51 */       throw new IOException("Failed to obtain InputStream from url: " + configURL);
/*     */     }
/*  53 */     List errors = new ArrayList();
/*  54 */     DocumentBuilderFactory docBuilderFactory = null;
/*  55 */     docBuilderFactory = DocumentBuilderFactory.newInstance();
/*  56 */     docBuilderFactory.setValidating(true);
/*  57 */     docBuilderFactory.setNamespaceAware(true);
/*     */     try
/*     */     {
/*  60 */       docBuilderFactory.setAttribute("http://apache.org/xml/features/validation/schema", Boolean.valueOf(true));
/*     */     }
/*     */     catch (IllegalArgumentException e) {
/*  63 */       docBuilderFactory.setValidating(false);
/*  64 */       docBuilderFactory.setNamespaceAware(false);
/*     */     }
/*  66 */     InputSource source = new InputSource(is);
/*  67 */     DocumentBuilder docBuilder = docBuilderFactory.newDocumentBuilder();
/*  68 */     docBuilder.setEntityResolver(resolver);
/*  69 */     docBuilder.setErrorHandler(new ErrorLogger("XML InputStream", errors, resolver));
/*  70 */     Document doc = docBuilder.parse(source);
/*  71 */     if (errors.size() != 0) {
/*  72 */       throw new PersistenceException("invalid persistence.xml", (Throwable)errors.get(0));
/*     */     }
/*  74 */     return doc;
/*     */   }
/*     */   
/*     */ 
/*     */   public static List<PersistenceMetadata> deploy(URL url, Map overrides, EntityResolver resolver)
/*     */     throws Exception
/*     */   {
/*  81 */     return deploy(url, overrides, resolver, PersistenceUnitTransactionType.JTA);
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */   public static List<PersistenceMetadata> deploy(URL url, Map overrides, EntityResolver resolver, PersistenceUnitTransactionType defaultTransactionType)
/*     */     throws Exception
/*     */   {
/*  89 */     Document doc = loadURL(url, resolver);
/*  90 */     Element top = doc.getDocumentElement();
/*  91 */     NodeList children = top.getChildNodes();
/*  92 */     ArrayList<PersistenceMetadata> units = new ArrayList();
/*  93 */     for (int i = 0; i < children.getLength(); i++) {
/*  94 */       if (children.item(i).getNodeType() == 1) {
/*  95 */         Element element = (Element)children.item(i);
/*  96 */         String tag = element.getTagName();
/*  97 */         if (tag.equals("persistence-unit")) {
/*  98 */           PersistenceMetadata metadata = parsePersistenceUnit(element);
/*     */           
/* 100 */           if (overrides.containsKey("javax.persistence.provider")) {
/* 101 */             String provider = (String)overrides.get("javax.persistence.provider");
/* 102 */             metadata.setProvider(provider);
/*     */           }
/* 104 */           if (overrides.containsKey("javax.persistence.transactionType")) {
/* 105 */             String transactionType = (String)overrides.get("javax.persistence.transactionType");
/* 106 */             metadata.setTransactionType(getTransactionType(transactionType));
/*     */           }
/* 108 */           if (overrides.containsKey("javax.persistence.jtaDataSource")) {
/* 109 */             String dataSource = (String)overrides.get("javax.persistence.jtaDataSource");
/* 110 */             metadata.setJtaDatasource(dataSource);
/*     */           }
/* 112 */           if (overrides.containsKey("javax.persistence.nonJtaDataSource")) {
/* 113 */             String dataSource = (String)overrides.get("javax.persistence.nonJtaDataSource");
/* 114 */             metadata.setNonJtaDatasource(dataSource);
/*     */           }
/*     */           
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/* 122 */           PersistenceUnitTransactionType transactionType = metadata.getTransactionType();
/* 123 */           Boolean isJTA = null;
/* 124 */           if (StringHelper.isNotEmpty(metadata.getJtaDatasource())) {
/* 125 */             isJTA = Boolean.TRUE;
/*     */           }
/* 127 */           else if (StringHelper.isNotEmpty(metadata.getNonJtaDatasource())) {
/* 128 */             isJTA = Boolean.FALSE;
/*     */           }
/* 130 */           if (transactionType == null) {
/* 131 */             if (isJTA == Boolean.TRUE) {
/* 132 */               transactionType = PersistenceUnitTransactionType.JTA;
/*     */             }
/* 134 */             else if (isJTA == Boolean.FALSE) {
/* 135 */               transactionType = PersistenceUnitTransactionType.RESOURCE_LOCAL;
/*     */             }
/*     */             else {
/* 138 */               transactionType = defaultTransactionType;
/*     */             }
/*     */           }
/* 141 */           metadata.setTransactionType(transactionType);
/* 142 */           Properties properties = metadata.getProps();
/* 143 */           ConfigurationHelper.overrideProperties(properties, overrides);
/* 144 */           units.add(metadata);
/*     */         }
/*     */       }
/*     */     }
/* 148 */     return units;
/*     */   }
/*     */   
/*     */   private static PersistenceMetadata parsePersistenceUnit(Element top) throws Exception
/*     */   {
/* 153 */     PersistenceMetadata metadata = new PersistenceMetadata();
/* 154 */     String puName = top.getAttribute("name");
/* 155 */     if (StringHelper.isNotEmpty(puName)) {
/* 156 */       log.trace("Persistent Unit name from persistence.xml: " + puName);
/* 157 */       metadata.setName(puName);
/*     */     }
/* 159 */     NodeList children = top.getChildNodes();
/* 160 */     for (int i = 0; i < children.getLength(); i++) {
/* 161 */       if (children.item(i).getNodeType() == 1) {
/* 162 */         Element element = (Element)children.item(i);
/* 163 */         String tag = element.getTagName();
/*     */         
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/* 170 */         if (tag.equals("non-jta-data-source")) {
/* 171 */           metadata.setNonJtaDatasource(XmlHelper.getElementContent(element));
/*     */         }
/* 173 */         else if (tag.equals("jta-data-source")) {
/* 174 */           metadata.setJtaDatasource(XmlHelper.getElementContent(element));
/*     */         }
/* 176 */         else if (tag.equals("provider")) {
/* 177 */           metadata.setProvider(XmlHelper.getElementContent(element));
/*     */         }
/* 179 */         else if (tag.equals("class")) {
/* 180 */           metadata.getClasses().add(XmlHelper.getElementContent(element));
/*     */         }
/* 182 */         else if (tag.equals("mapping-file")) {
/* 183 */           metadata.getMappingFiles().add(XmlHelper.getElementContent(element));
/*     */         }
/* 185 */         else if (tag.equals("jar-file")) {
/* 186 */           metadata.getJarFiles().add(XmlHelper.getElementContent(element));
/*     */         }
/* 188 */         else if (tag.equals("exclude-unlisted-classes")) {
/* 189 */           metadata.setExcludeUnlistedClasses(true);
/*     */         }
/* 191 */         else if (tag.equals("properties")) {
/* 192 */           NodeList props = element.getChildNodes();
/* 193 */           for (int j = 0; j < props.getLength(); j++) {
/* 194 */             if (props.item(j).getNodeType() == 1) {
/* 195 */               Element propElement = (Element)props.item(j);
/* 196 */               if ("property".equals(propElement.getTagName())) {
/* 197 */                 String propName = propElement.getAttribute("name").trim();
/* 198 */                 String propValue = propElement.getAttribute("value").trim();
/* 199 */                 if (StringHelper.isEmpty(propValue))
/*     */                 {
/* 201 */                   propValue = XmlHelper.getElementContent(propElement, "");
/*     */                 }
/* 203 */                 metadata.getProps().put(propName, propValue);
/*     */               }
/*     */             }
/*     */           }
/*     */         }
/*     */       }
/*     */     }
/* 210 */     PersistenceUnitTransactionType transactionType = getTransactionType(top.getAttribute("transaction-type"));
/* 211 */     if (transactionType != null) { metadata.setTransactionType(transactionType);
/*     */     }
/* 213 */     return metadata;
/*     */   }
/*     */   
/*     */   public static PersistenceUnitTransactionType getTransactionType(String elementContent) {
/* 217 */     if (StringHelper.isEmpty(elementContent)) {
/* 218 */       return null;
/*     */     }
/* 220 */     if (elementContent.equalsIgnoreCase("JTA")) {
/* 221 */       return PersistenceUnitTransactionType.JTA;
/*     */     }
/* 223 */     if (elementContent.equalsIgnoreCase("RESOURCE_LOCAL")) {
/* 224 */       return PersistenceUnitTransactionType.RESOURCE_LOCAL;
/*     */     }
/*     */     
/* 227 */     throw new PersistenceException("Unknown TransactionType: " + elementContent);
/*     */   }
/*     */   
/*     */   public static class ErrorLogger implements ErrorHandler
/*     */   {
/*     */     private String file;
/*     */     private List errors;
/*     */     private EntityResolver resolver;
/*     */     
/*     */     ErrorLogger(String file, List errors, EntityResolver resolver) {
/* 237 */       this.file = file;
/* 238 */       this.errors = errors;
/* 239 */       this.resolver = resolver;
/*     */     }
/*     */     
/*     */     public void error(SAXParseException error) {
/* 243 */       if (((this.resolver instanceof EJB3DTDEntityResolver)) && 
/* 244 */         (!((EJB3DTDEntityResolver)this.resolver).isResolved())) { return;
/*     */       }
/* 246 */       PersistenceXmlLoader.log.error("Error parsing XML: " + this.file + '(' + error.getLineNumber() + ") " + error.getMessage());
/* 247 */       this.errors.add(error);
/*     */     }
/*     */     
/*     */     public void fatalError(SAXParseException error) {
/* 251 */       PersistenceXmlLoader.log.error("Error parsing XML: " + this.file + '(' + error.getLineNumber() + ") " + error.getMessage());
/* 252 */       this.errors.add(error);
/*     */     }
/*     */     
/*     */     public void warning(SAXParseException warn) {
/* 256 */       PersistenceXmlLoader.log.warn("Warning parsing XML: " + this.file + '(' + warn.getLineNumber() + ") " + warn.getMessage());
/*     */     }
/*     */   }
/*     */ }


/* Location:              C:\Users\anandkum\Desktop\source\hibernate-entitymanager-3.3.2.GA.jar!\org\hibernate\ejb\packaging\PersistenceXmlLoader.class
 * Java compiler version: 5 (49.0)
 * JD-Core Version:       0.7.1
 */